/////////////////////////////////////////////////////////////////////////////
// Name:        ax_menu_speech_settings.h
// Purpose:     Speech settings menu class
// Author:      Julian Smart
// Modified by:
// Created:     2009-03-20
// RCS-ID:
// Copyright:   (c) Julian Smart
// Licence:     New BSD License
/////////////////////////////////////////////////////////////////////////////

#ifndef _AX_MENU_SPEECH_SETTINGS_H_
#define _AX_MENU_SPEECH_SETTINGS_H_

/*
 * Includes
 */

#include "ax/ax_activator.h"
#include "ax/ax_menu.h"

class AxSpeech;
class AxChoiceMenuItem;
class wxSpinEvent;

/**
    @class AxSpeechSettingsMenuItem

    @category{ax}
 */

class AxSpeechSettingsMenuItem: public AxMenuItem
{
    DECLARE_CLASS( AxSpeechSettingsMenuItem )
    DECLARE_EVENT_TABLE( )

public:
// Constructors

    AxSpeechSettingsMenuItem(const AxSpeechSettingsMenuItem& item)
    { Copy(item); m_speech = item.m_speech; m_controller = item.m_controller; }
    AxSpeechSettingsMenuItem(AxMenuController* controller, AxSpeech* speech);

    void OnSelectEngine(wxCommandEvent& event);
    void OnSelectVoice(wxCommandEvent& event);
    void OnSelectVolume(wxSpinEvent& event);
    void OnSelectSpeed(wxSpinEvent& event);
    void OnSelectMultipleVoices(wxCommandEvent& event);

    void CheckCompatibleVoices();

    virtual AxMenuItem* Clone() { return new AxSpeechSettingsMenuItem(* this); }

protected:
    AxSpeech*           m_speech;
    AxMenuController*   m_controller;
};

/**
    @class AxSpeechSettingsActivator

    Top-level item activator for the speech settings,
    creating the actual menu on demand.

    @category{ax}

 */

class AxSpeechSettingsActivator: public AxActivator
{
public:
    
    DECLARE_DYNAMIC_CLASS( AxSpeechSettingsActivator )

public:
// Constructors

    AxSpeechSettingsActivator() { Init(); }

    AxSpeechSettingsActivator(AxSpeech* speech) { m_speech = speech; }

    AxSpeechSettingsActivator(const AxSpeechSettingsActivator& activator) { Init(); Copy(activator); }

    void Init() { m_speech = NULL; m_mainVoicesItem = NULL; m_contentVoicesItem = NULL; }

    void Copy(const AxSpeechSettingsActivator& activator)
    {
        AxActivator::Copy(activator);
        m_speech = activator.m_speech;
        m_mainVoicesItem = activator.m_mainVoicesItem;
        m_contentVoicesItem = activator.m_contentVoicesItem;
    }

    virtual AxActivator* Clone() const { return new AxSpeechSettingsActivator(* this); }

    /// Called when the user activates a menu item by pressing return or double-clicking
    virtual AxMenu* Activate(AxMenuController* controller, AxMenuItem* item);

    AxChoiceMenuItem* GetVoicesItem(int channel) const { return channel == 0 ? m_mainVoicesItem : m_contentVoicesItem; }

protected:
    AxSpeech*           m_speech;
    AxChoiceMenuItem*   m_mainVoicesItem;
    AxChoiceMenuItem*   m_contentVoicesItem;
};

#endif
    // _AX_MENU_SPEECH_SETTINGS_H_
