/////////////////////////////////////////////////////////////////////////////
// Name:        ax_mp3info.h
// Purpose:     MP3 info class
// Author:      Roman Nurik, Julian Smart
// Modified by:
// Created:     2009-01-03
// RCS-ID:      $Id$
// Copyright:   (c) Roman Nurik, Julian Smart
//              Released under the New BSD License
//              with kind permission from Roman Nurik.
// Licence:     New BSD License
/////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////                  Copyright (C) 2002, Roman Nurik                       ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#ifndef _AX_MP3INFO_H_
#define _AX_MP3INFO_H_

// list of genres
static char* g_arrMP3Genre[] = {
	"Blues","Classic Rock","Country","Dance","Disco","Funk","Grunge","Hip-Hop",
	"Jazz","Metal","New Age","Oldies","Other","Pop","R&B","Rap","Reggae","Rock",
	"Techno","Industrial","Alternative","Ska","Death Metal","Pranks","Soundtrack",
	"Euro-Techno","Ambient","Trip Hop","Vocal","Jazz Funk","Fusion","Trance",
	"Classical","Instrumental","Acid","House","Game","Sound Clip","Gospel","Noise",
	"Alternative Rock","Bass","Soul","Punk","Space","Meditative","Instrumental Pop",
	"Instrumental Rock","Ethnic","Gothic","Darkwave","Techno-Industrial","Electronic",
	"Pop Folk","Eurodance","Dream","Southern Rock","Comedy","Cult","Gangsta","Top 40",
	"Christian Rap","Pop Funk","Jungle","Native American","Cabaret","New Wave",
	"Psychadelic","Rave","ShowTunes","Trailer","Lo-Fi","Tribal","Acid Punk","Acid Jazz",
	"Polka","Retro","Musical","Rock & Roll","Hard Rock","Folk","Folk Rock",
	"National Folk","Swing","Fast Fusion","Bebob","Latin","Revival","Celtic",
	"Bluegrass","Avantgarde","Gothic Rock","Progressive Rock","Psychedelic Rock",
	"Symphonic Rock","Slow Rock","Big Band","Chorus","Easy Listening","Acoustic",
	"Humour","Speech","Chanson","Opera","Chamber Music","Sonata","Symphony","Booty Bass",
	"Primus","Porn Groove","Satire","Slow Jam","Club","Tango","Samba","Folklore","Ballad",
	"Power Ballad","Rhytmic Soul","Freestyle","Duet","Punk Rock","Drum Solo","A Capella",
	"Euro House","Dance Hall","Goa","Drum & Bass","Club House","Hardcore","Terror",
	"Indie","BritPop","Negerpunk","Polsk Punk","Beat","Christian Gangsta Rap",
	"Heavy Metal","Black Metal","Crossover","Contemporary Christian","Christian Rock",
	"Merengue","Salsa","Trash Metal","Anime","JPop","SynthPop"
};

static int g_nMP3GenreCount = 148;

// matrix of bitrates [based on MPEG version data][bitrate index]
static int g_nMP3BitRate[6][16] = {
	{0,32,64,96,128,160,192,224,256,288,320,352,384,416,448,-1},
	{0,32,48,56, 64, 80, 96,112,128,160,192,224,256,320,384,-1},
	{0,32,40,48, 56, 64, 80, 96,112,128,160,192,224,256,320,-1},
	{0,32,48,56, 64, 80, 96,112,128,144,160,176,192,224,256,-1},
	{0, 8,16,24, 32, 40, 48, 56, 64, 80, 96,112,128,144,160,-1},
	{0, 8,16,24, 32, 64, 80, 56, 64,128,160,112,128,256,320,-1},
};

// MP3FRAMEHEADER structure
struct MP3FRAMEHEADER
{
	unsigned emphasis : 2;			// M
	unsigned original : 1;			// L
	unsigned copyright : 1;			// K
	unsigned modeext : 2;			// J
	unsigned chanmode : 2;			// I
	unsigned privbit : 1;			// H
	unsigned padding : 1;			// G
	unsigned samplerate : 2;		// F
	unsigned bitrate : 4;			// E
	unsigned hascrc : 1;			// D
	unsigned mpeglayer : 2;			// C
	unsigned mpegver : 2;			// B
	unsigned framesync : 11;		// A
};

// MP3ID3V1TAG structure
struct MP3ID3V1TAG
{
	char ident[3]; // TAG
	char title[30];
	char artist[30];
	char album[30];
	char year[4];
	char comment[28];
	wxByte reserved;
	wxByte tracknum;
	wxByte genre;
};

// ENMPEGVER enumeration
enum ENMPEGVER
{
	MPEGVER_NA,		// reserved, N/A
	MPEGVER_25,		// 2.5
	MPEGVER_2,		// 2.0 (ISO/IEC 13818-3)
	MPEGVER_1		// 1.0 (ISO/IEC 11172-3)
};

// ENCHANNELMODE enumeration
enum ENCHANNELMODE
{
	MP3CM_STEREO,
	MP3CM_JOINT_STEREO,
	MP3CM_DUAL_CHANNEL,
	MP3CM_SINGLE_CHANNEL
};

// ENEMPHASIS enumeration
enum ENEMPHASIS
{
	MP3EM_NONE,
	MP3EM_50_15_MS,		// 50/15 ms
	MP3EM_RESERVED,
	MP3EM_CCIT_J17		// CCIT J.17
};

// bitmask of validity of files
#define MP3VLD_INVALID			0x0000
#define MP3VLD_ID3V1_VALID		0x0001
#define MP3VLD_DATA_VALID		0x0002

// AxMP3Info class
class AxMP3Info
{
private:
	wxString m_strFile;
	wxUint32 m_dwValidity;

	// ID3V1 information
	wxString m_strArtist;
	wxString m_strTitle;
	wxString m_strAlbum;
	wxString m_strComment;
	wxString m_strYear;
	int m_nTrack;
	int m_nGenre;

	// MP3 frame information
	int m_nFrames;
	int m_nLength;					// in seconds
	ENMPEGVER m_enMPEGVersion;
	int m_nMPEGLayer;
	bool m_bHasCRC;
	int m_nBitRate;					// average if VBR, 0 if "free"
	int m_nSampleRate;
	ENCHANNELMODE m_enChannelMode;
	ENEMPHASIS m_enEmphasis;
	bool m_bCopyrighted;
	bool m_bOriginal;

public:
	AxMP3Info(wxString strFile = wxEmptyString);
	virtual ~AxMP3Info();

// operations
	bool Open(wxString strFile);
	// bool WriteID3V1(wxString strDestFile = wxEmptyString, bool bClear = FALSE); // defaults to current file
	void Clear();

	void SetArtist(wxString strArtist){ m_strArtist = strArtist; }
	void SetTitle(wxString strTitle){ m_strTitle = strTitle; }
	void SetAlbum(wxString strAlbum){ m_strAlbum = strAlbum; }
	void SetComment(wxString strComment){ m_strComment = strComment; }
	void SetYear(wxString strYear){ m_strYear = strYear; }
	void SetTrack(int nTrack){ m_nTrack = nTrack; }
	void SetGenreIndex(int nGenre){ m_nGenre = nGenre; }

// attributes
	wxString GetFileName() const { return m_strFile; }
	wxUint32 GetValidity() const { return m_dwValidity; }

	wxString GetArtist() const { return m_strArtist; }
	wxString GetTitle() const { return m_strTitle; }
	wxString GetAlbum() const { return m_strAlbum; }
	wxString GetComment() const { return m_strComment; }
	wxString GetYear() const { return m_strYear; }
	int GetTrack() const { return m_nTrack; }
	int GetGenreIndex() const { return m_nGenre; }

	int GetNumFrames() const { return m_nFrames; }
	int GetLength() const { return m_nLength; }
	ENMPEGVER GetMPEGVersion() const { return m_enMPEGVersion; }
	int GetMPEGLayer() const { return m_nMPEGLayer; }
	bool HasCRC() const { return m_bHasCRC; }
	int GetBitRate() const { return m_nBitRate; }
	int GetSampleRate() const { return m_nSampleRate; }
	ENCHANNELMODE GetChannelMode() const { return m_enChannelMode; }
	ENEMPHASIS GetEmphasis() const { return m_enEmphasis; }
	bool IsCopyrighted() const { return m_bCopyrighted; }
	bool IsOriginal() const { return m_bOriginal; }

// functions
	static wxString GetGenreString(int nIndex);
	static wxString GetLengthString(int nSeconds);

private:
// helper functions
	bool GetNextFrameHeader(FILE* hFile, MP3FRAMEHEADER* pHeader, int nPassBytes = 0);
	void ChangeEndian(void* pBuffer, int nBufSize);
};

#endif
    // _AX_MP3INFO_H_
