/////////////////////////////////////////////////////////////////////////////
// Name:        tts_macosx.h
// Purpose:     A wxTextToSpeech handler for the Mac OS X Speech Manager
// Author:      Julian Smart
// Modified by: 
// Created:     2009-02-10
// RCS-ID:      
// Copyright:   (c) Julian Smart
// Licence:     New BSD License
/////////////////////////////////////////////////////////////////////////////

#ifndef _WX_TTS_MACOSX_
#define _WX_TTS_MACOSX_

/*
 * Includes
 */

#include "tts/tts.h"

/**
    @class wxTTSMacOSXHandler
    
    @category{tts}
 */

class wxTTSMacOSXData;
class wxTTSMacOSXHandler: public wxTTSHandler
{    
    DECLARE_DYNAMIC_CLASS( wxTTSMacOSXHandler )

public:
// Constructors

    wxTTSMacOSXHandler();
    ~wxTTSMacOSXHandler();

// Operations

    /// Member initialisation
    void Init();

    /// Initialize the engine. Can call multiple times.
    virtual bool Initialize();

    /// Clean up the engine. Can call multiple times.
    virtual bool Uninitialize();

    /// Speak the text
    virtual bool Speak(const wxString& text, int options = wxTTS_SPEAK_ASYNC);

    /// Speak the file
    virtual bool SpeakFile(const wxString& filename, int options = wxTTS_SPEAK_ASYNC, const wxString& waveFilename = wxEmptyString);

    /// Speak the text or file
    virtual bool DoSpeak(const wxString& textOrFilename, bool isFilename, int options = wxTTS_SPEAK_ASYNC, const wxString& waveFilename = wxEmptyString);

    /// Is it speaking?
    virtual bool IsSpeaking() const;

    /// Is it paused?
    virtual bool IsPaused() const;

    /// Stop speaking
    virtual bool Stop();

    /// Pause
    virtual bool Pause();

    /// Resume
    virtual bool Resume();

    /// Get a list of the available voices
    virtual wxArrayString GetAvailableVoices() const;

    /// Get a list of the available voice variants
    virtual wxArrayString GetAvailableVoiceVariants() const;

    /// Does this handler have the specified capability?
    virtual bool HasCapability(wxTTSCapability capability) const;

    virtual int GetMinSpeed() const { return 80; }
    virtual int GetMaxSpeed() const { return 390; }
    virtual int GetMinVolume() const { return 0; }
    virtual int GetMaxVolume() const { return 100; }

    // Native pitch control is a multiplication factor,
    // but convert to percentage for wxTextToSpeech.
    // 50 would be the default.
    virtual int GetMinPitch() const { return 0; }
    virtual int GetMaxPitch() const { return 100; }

    /// Get the default voice
    virtual wxString GetDefaultVoice() const;

    /// Get the default voice variant
    virtual wxString GetDefaultVoiceVariant() const;

    /// Get data
    wxTTSMacOSXData* GetData() const { return m_data; }

protected:

    wxTTSMacOSXData* m_data;
};

#endif
    // _WX_TTS_MACOSX_
